Imports System.Runtime.InteropServices
Imports System.Drawing
Imports ESRI.ArcGIS.ADF.BaseClasses
Imports ESRI.ArcGIS.ADF.CATIDs
Imports ESRI.ArcGIS.Framework
Imports ESRI.ArcGIS.ArcMapUI
Imports ESRI.ArcGIS.Carto

<ComClass(AddLayerCommand.ClassId, AddLayerCommand.InterfaceId, AddLayerCommand.EventsId), _
 ProgId("GPSLayer.AddLayerCommand")> _
Public NotInheritable Class AddLayerCommand
   Inherits BaseCommand

#Region "COM GUIDs"
   ' These  GUIDs provide the COM identity for this class 
   ' and its COM interfaces. If you change them, existing 
   ' clients will no longer be able to access the class.
   Public Const ClassId As String = "1220bed4-185e-44eb-900f-3476729c1412"
   Public Const InterfaceId As String = "1c40b5a6-b95f-4cd3-8afe-7f1f0f20031d"
   Public Const EventsId As String = "12c61582-6205-454d-902b-c6042e132ba3"
#End Region

#Region "COM Registration Function(s)"
   <ComRegisterFunction(), ComVisibleAttribute(False)> _
   Public Shared Sub RegisterFunction(ByVal registerType As Type)
      ' Required for ArcGIS Component Category Registrar support
      ArcGISCategoryRegistration(registerType)

      'Add any COM registration code after the ArcGISCategoryRegistration() call

   End Sub

   <ComUnregisterFunction(), ComVisibleAttribute(False)> _
   Public Shared Sub UnregisterFunction(ByVal registerType As Type)
      ' Required for ArcGIS Component Category Registrar support
      ArcGISCategoryUnregistration(registerType)

      'Add any COM unregistration code after the ArcGISCategoryUnregistration() call

   End Sub

#Region "ArcGIS Component Category Registrar generated code"
   Private Shared Sub ArcGISCategoryRegistration(ByVal registerType As Type)
      Dim regKey As String = String.Format("HKEY_CLASSES_ROOT\CLSID\{{{0}}}", registerType.GUID)
      MxCommands.Register(regKey)

   End Sub
   Private Shared Sub ArcGISCategoryUnregistration(ByVal registerType As Type)
      Dim regKey As String = String.Format("HKEY_CLASSES_ROOT\CLSID\{{{0}}}", registerType.GUID)
      MxCommands.Unregister(regKey)

   End Sub

#End Region
#End Region


   Private m_application As IApplication

   ' A creatable COM class must have a Public Sub New() 
   ' with no parameters, otherwise, the class will not be 
   ' registered in the COM registry and cannot be created 
   ' via CreateObject.
   Public Sub New()
      MyBase.New()

      MyBase.m_category = "GPS"  'localizable text 
      MyBase.m_caption = "Add GPS Layer"   'localizable text 
      MyBase.m_message = ""   'localizable text 
      MyBase.m_toolTip = "Add GPS Layer" 'localizable text 
      MyBase.m_name = "GPS_AddGPSLayer"  'unique id, non-localizable (e.g. "MyCategory_ArcMapCommand")

      Try
         Dim bitmapResourceName As String = Me.GetType().Name + ".bmp"
         MyBase.m_bitmap = New Bitmap(Me.GetType(), bitmapResourceName)
      Catch ex As Exception
         System.Diagnostics.Trace.WriteLine(ex.Message, "Invalid Bitmap")
      End Try


   End Sub


   Public Overrides Sub OnCreate(ByVal hook As Object)
      If Not hook Is Nothing Then
         m_application = CType(hook, IApplication)

         'Disable if it is not ArcMap
         If TypeOf hook Is IMxApplication Then
            MyBase.m_enabled = True
         Else
            MyBase.m_enabled = False
         End If
      End If

   End Sub

   Public Overrides Sub OnClick()

      ' Check if layer already exists

      Dim pDoc As IMxDocument
      Dim pMap As IMap
      Dim pLayers As IEnumLayer
      Dim pLayer As ILayer

      pDoc = m_application.Document
      pMap = pDoc.FocusMap
      pLayers = pMap.Layers(Nothing, True)
      pLayers.Reset()
      pLayer = pLayers.Next
      Do While Not pLayer Is Nothing
         If TypeOf pLayer Is IGPSLayer Then
            MsgBox("Layer already added", MsgBoxStyle.Exclamation)
            Exit Sub
         End If
         pLayer = pLayers.Next
      Loop

      ' Add layer

      Dim pGPSLayer As GPSLayerClass

      pGPSLayer = New GPSLayerClass
      pMap.AddLayer(pGPSLayer)

   End Sub

End Class



