Imports System.Runtime.InteropServices
Imports System.Threading
Imports System.Windows.Forms
Imports ESRI.ArcGIS.Carto
Imports ESRI.ArcGIS.Display
Imports ESRI.ArcGIS.esriSystem
Imports ESRI.ArcGIS.Geometry

Public Class GPSControllerForm

	Private m_pLayer As IGPSLayer
   Private m_bEnabled As Boolean
   Private m_pFeedMan As IRealTimeFeedManager
   Private WithEvents m_GPSFeed As GpsFeed
	Private m_pWGS84 As IGeographicCoordinateSystem
   Private m_pMapPoint As IPoint
   Private m_pPrevPoint As IPoint

   Private m_pSD As IScreenDisplay
   Private m_pSR As ISpatialReference
   Private WithEvents m_pDisplayEvents As ScreenDisplay
   Private m_bPanToPoint As Boolean
	Private m_dPtSize As Double
	Private m_pMapSymbol As IMarkerSymbol = Nothing

	Public Sub SetLayer(ByVal pLayer As IGPSLayer)
		m_pLayer = pLayer
	End Sub

	Public Sub SetMarkerSymbol(ByVal pMarkerSymbol As IMarkerSymbol)
		m_pMapSymbol = pMarkerSymbol
	End Sub

	Public Sub SetDisplay(ByVal pDisplay As IDisplay)

		' The display should never change during the life of this dialog

		m_pSD = Nothing
		m_pDisplayEvents = Nothing
		GC.Collect()
		m_pSD = pDisplay
		m_pDisplayEvents = pDisplay

	End Sub

	Public Sub SetSpatialRef(ByVal pSR As ISpatialReference)
		m_pSR = Nothing
		GC.Collect()
		m_pSR = pSR
	End Sub

	Public Sub ShutDown()

		Dim pRTF As IRealTimeFeed

		pRTF = m_GPSFeed
		If Not pRTF Is Nothing Then
			Try
				pRTF.Stop()
			Catch ex As Exception
				' Nothing to do
			End Try
			pRTF = Nothing
			m_GPSFeed = Nothing
			m_pFeedMan = Nothing
			' Marshal.ReleaseComObject(m_GPSFeed)
			' Marshal.ReleaseComObject(m_pFeedMan)
		End If
		m_bEnabled = False
		btnStart.Enabled = True
		btnStop.Enabled = False
		btnDetect.Enabled = True
		If Not m_pPrevPoint Is Nothing Then
			UndrawPoint(m_pPrevPoint)
		End If
		If Not m_pMapPoint Is Nothing Then
			UndrawPoint(m_pMapPoint)
		End If
		m_pMapPoint = Nothing
		m_pPrevPoint = Nothing
		lblLon.Text = "0"
		lblLat.Text = "0"
		lblStatus.Text = "N/A"
		lblPDOP.Text = "N/A"
		lblSatCount.Text = "N/A"
		GC.Collect()

	End Sub

	Private Sub GPSConfigForm_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load

		m_dPtSize = 22

		' Populate available COM ports

		Dim sP As String

		cbComPort.Items.Clear()
		For Each sP In My.Computer.Ports.SerialPortNames
			cbComPort.Items.Add(sP)
		Next
		cbComPort.SelectedIndex = 0

	End Sub

	Private Sub GPSControllerForm_FormClosing(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles Me.FormClosing
		ShutDown()
	End Sub

	Private Sub chkPan_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles chkPan.CheckedChanged
		m_bPanToPoint = chkPan.Checked
	End Sub

	Private Sub btnDetect_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnDetect.Click

		Dim sP As String
		Dim bFound As Boolean

		btnDetect.Enabled = False
		btnStart.Enabled = False
		bFound = False
		For Each sP In My.Computer.Ports.SerialPortNames
			bFound = TestComPort(sP)
			If bFound Then
				MsgBox("GPS found on " & sP & ".", MsgBoxStyle.Information)
				cbComPort.Text = sP
				btnDetect.Enabled = True
				btnStart.Enabled = True
				Exit Sub
			End If
		Next
		MsgBox("GPS port not found.", MsgBoxStyle.Exclamation)
		btnDetect.Enabled = True
		btnStart.Enabled = True

	End Sub

	Private Sub btnSymbol_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnSymbol.Click
		Dim sf As New SymbolForm
		If sf.ShowDialog() <> Windows.Forms.DialogResult.OK Then
			Exit Sub
		End If
		m_pMapSymbol = sf.MarkerSymbol ' This will be nothing if resetting symbol
		m_pLayer.SetSymbol(m_pMapSymbol)
	End Sub

	Private Sub btnStart_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnStart.Click

		m_bEnabled = False
		Try
			InitGPSFeed()
		Catch ex As Exception
			MsgBox("Error initializing GPS feed.", MsgBoxStyle.Exclamation)
			m_GPSFeed = Nothing
			m_pFeedMan = Nothing
			GC.Collect()
			Exit Sub
		End Try
		btnStart.Enabled = False
		btnStop.Enabled = True
		btnDetect.Enabled = True
		m_bEnabled = True

	End Sub

	Private Sub btnStop_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnStop.Click
		ShutDown()
	End Sub

	Private Sub m_GPSFeed_PositionUpdated(ByRef position As ESRI.ArcGIS.Carto.esriGpsPositionInfo, ByVal estimate As Boolean) Handles m_GPSFeed.PositionUpdated
		FeedEvent()
	End Sub

	Private Sub m_pDisplayEvents_DisplayInvalidated(ByVal Display As ESRI.ArcGIS.Display.IDisplay, ByVal rect As ESRI.ArcGIS.Geometry.IEnvelope, ByVal [erase] As Boolean, ByVal cacheID As Short) Handles m_pDisplayEvents.DisplayInvalidated
		m_bEnabled = False
	End Sub

	Private Sub m_pDisplayEvents_DisplayScrolled(ByVal Display As ESRI.ArcGIS.Display.IDisplay, ByVal deltaX As Integer, ByVal deltaY As Integer) Handles m_pDisplayEvents.DisplayScrolled
		m_bEnabled = False
	End Sub

	Private Sub m_pDisplayEvents_DisplayStarted(ByVal Display As ESRI.ArcGIS.Display.IDisplay) Handles m_pDisplayEvents.DisplayStarted
		m_bEnabled = False
	End Sub

	Private Sub m_pDisplayEvents_DisplayFinished(ByVal Display As ESRI.ArcGIS.Display.IDisplay) Handles m_pDisplayEvents.DisplayFinished
		m_bEnabled = True
	End Sub

	Private Sub InitGPSFeed()

		Dim pFeed As IGpsFeed
		Dim pConn As IGpsConnection
		Dim sP As String
		Dim iPort As Short
		Dim pRTF As IRealTimeFeed


		m_pFeedMan = New RealTimeFeedManagerClass
		m_GPSFeed = m_pFeedMan.GpsFeed
		pFeed = m_GPSFeed
		pConn = pFeed.Connection
		sP = cbComPort.Text
		iPort = CShort(Strings.Right(sP, sP.Length - 3))
		pConn.CommPort = iPort
		pConn.BaudRate = 4800
		pConn.DataBits = 8
		pConn.Parity = esriGpsParity.esriGpsParityNone
		pConn.StopBits = esriGpsStopBits.esriGpsStopBits1
		pRTF = pFeed
		pRTF.Start()

	End Sub

	Private Sub FeedEvent()

		If Not m_bEnabled Then
			Exit Sub
		End If
		m_bEnabled = False

		Dim bSuccess As Boolean
		Dim pPropset As IPropertySet
		Dim dLon As Double, dLat As Double

		pPropset = GetGPSData()
		UpdateItems(pPropset)
		bSuccess = pPropset.GetProperty("Success")
		If Not bSuccess Then
			m_bEnabled = True
			Exit Sub
		End If
		dLat = pPropset.GetProperty("Lat")
		dLon = pPropset.GetProperty("Lon")
		DrawGPSPoint(dLon, dLat)
		m_bEnabled = True

	End Sub

	Private Function GetGPSData() As IPropertySet

		Dim pPropSet As IPropertySet
		Dim dLat As Double, dLon As Double
		Dim sStatus As String, sPDOP As String, sSatCount As String


		pPropSet = New PropertySet
		dLon = CDbl(lblLon.Text)
		dLat = CDbl(lblLat.Text)
		sStatus = lblStatus.Text
		sPDOP = lblPDOP.Text
		sSatCount = lblSatCount.Text
		pPropSet.SetProperty("Success", False)
		pPropSet.SetProperty("Lat", dLat)
		pPropSet.SetProperty("Lon", dLon)
		pPropSet.SetProperty("Status", sStatus)
		pPropSet.SetProperty("PDOP", sPDOP)
		pPropSet.SetProperty("SatCount", sSatCount)

		Dim bValid As Boolean

		Dim pFeed As IRealTimeFeed
		Dim eDOPInfo As esriGpsDOPInfo
		Dim iSatCount As Long, iSatsUsed As Long
		Dim ePosInfo As esriGpsPositionInfo

		Try
			pFeed = m_GPSFeed
			eDOPInfo = pFeed.CurrentDopInfo
			iSatCount = pFeed.CurrentSatelliteCount
			ePosInfo = pFeed.CurrentPosition
			iSatsUsed = ePosInfo.satellitesInUse
		Catch ex As Exception
			sStatus = "Could not read data."
			pPropSet.SetProperty("Status", sStatus)
			Return pPropSet
		End Try

		bValid = (ePosInfo.pos2dValid <> 0)
		If Not bValid Then
			sPDOP = "N/A"
			sSatCount = "N/A"
		Else
			sPDOP = CStr(eDOPInfo.pdop)
			sSatCount = CStr(iSatsUsed) & " / " & CStr(iSatCount)
		End If
		dLon = ePosInfo.longitude
		dLat = ePosInfo.latitude
		sStatus = String.Format("{0:D2}:{1:D2}:{2:D2}", CInt(ePosInfo.hours), CInt(ePosInfo.minutes), CInt(ePosInfo.seconds))
		If bValid Then
			sStatus = sStatus & "; Valid"
		Else
			sStatus = sStatus & "; Warning"
		End If

		pPropSet.SetProperty("Success", True)
		pPropSet.SetProperty("Lat", dLat)
		pPropSet.SetProperty("Lon", dLon)
		pPropSet.SetProperty("Status", sStatus)
		pPropSet.SetProperty("PDOP", sPDOP)
		pPropSet.SetProperty("SatCount", sSatCount)
		Return pPropSet

	End Function

	Private Sub DrawGPSPoint(ByVal dLon As Double, ByVal dLat As Double)

		' Set up spatial ref

		Dim pSRF As ISpatialReferenceFactory

		If m_pWGS84 Is Nothing Then
			pSRF = New SpatialReferenceEnvironment
			m_pWGS84 = pSRF.CreateGeographicCoordinateSystem(esriSRGeoCSType.esriSRGeoCS_WGS1984)
		End If

		' Build point

		Dim pMapPoint As IPoint
		Dim pGeom As IGeometry

		If m_pSR Is Nothing Then
			Exit Sub
		End If
		pMapPoint = New Point
		pMapPoint.PutCoords(dLon, dLat)
		pGeom = pMapPoint
		pGeom.SpatialReference = m_pWGS84
		pMapPoint.Project(m_pSR)

		' Check for previous point

		If Not m_pMapPoint Is Nothing Then
			m_pPrevPoint = m_pMapPoint
		End If
		m_pMapPoint = pMapPoint

		' Check if point is outside map bounds

		Dim pDisp As IDisplayTransformation
		Dim pEnv As IEnvelope
		Dim pRel As IRelationalOperator

		If m_pSD Is Nothing Then
			Exit Sub
		End If
		pDisp = m_pSD.DisplayTransformation
		pEnv = pDisp.VisibleBounds
		pRel = pEnv
		If Not pRel.Contains(m_pMapPoint) Then
			If m_bPanToPoint Then
				m_pPrevPoint = Nothing
				pEnv.CenterAt(m_pMapPoint)
				pDisp.VisibleBounds = pEnv
				m_pSD.Invalidate(Nothing, True, esriScreenCache.esriAllScreenCaches)
			End If
			Exit Sub
		End If

		' Set up symbol

		Dim pFont As stdole.IFontDisp
		Dim pColor As IRgbColor
		Dim pCMSymbol As ICharacterMarkerSymbol

		If m_pMapSymbol Is Nothing Then
			pFont = New stdole.StdFont
			pFont.Name = "ESRI Default Marker"
			pFont.Size = m_dPtSize
			pColor = New RgbColor
			pColor.Red = 255
			pColor.Green = 0
			pColor.Blue = 0
			pCMSymbol = New CharacterMarkerSymbol
			With pCMSymbol
				.Angle = 0
				.CharacterIndex = 46
				.Color = pColor
				.Font = pFont
				.Size = m_dPtSize
				.XOffset = 0
				.YOffset = 0
			End With
			m_pMapSymbol = pCMSymbol
		End If

		' Draw point
		' Note:  if the map has a reference scale set, 
		' the point will be scaled relative to that

		If Not m_pPrevPoint Is Nothing Then
			UndrawPoint(m_pPrevPoint)
			m_pPrevPoint = Nothing
			GC.Collect()
			Application.DoEvents() ' This is necessary to allow the undraw to complete
		End If
		With m_pSD
			.StartDrawing(0, esriScreenCache.esriNoScreenCache)
			.SetSymbol(m_pMapSymbol)
			.DrawPoint(m_pMapPoint)
			.FinishDrawing()
		End With

	End Sub

	Private Sub UpdateItems(ByVal pPropSet As IPropertySet)

		' Update dialog items

		Dim dLat As Double, dLon As Double
		Dim sLat As String, sLon As String, sStatus As String
		Dim sPDOP As String, sSatCount As String

		dLat = pPropSet.GetProperty("Lat")
		dLon = pPropSet.GetProperty("Lon")
		sStatus = pPropSet.GetProperty("Status")
		sPDOP = pPropSet.GetProperty("PDOP")
		sSatCount = pPropSet.GetProperty("SatCount")
		sLat = String.Format("{0:F6}", dLat)
		sLon = String.Format("{0:F6}", dLon)
		lblLat.Text = sLat
		lblLat.Refresh()
		lblLon.Text = sLon
		lblLon.Refresh()
		lblPDOP.Text = sPDOP
		lblPDOP.Refresh()
		lblSatCount.Text = sSatCount
		lblSatCount.Refresh()
		lblStatus.Text = sStatus
		lblStatus.Refresh()
		If sStatus = "N/A" Then
			Exit Sub
		End If

	End Sub

	Private Sub UndrawPoint(ByVal pPoint As IPoint)

		Dim pGeom As IGeometry

		pGeom = pPoint
		If pGeom.IsEmpty Then
			Exit Sub
		End If

		' Calculate extent of symbol to invalidate

		Dim pDisp As IDisplayTransformation
		Dim dPtSize As Double, dRefScale As Double, dMapScale As Double

		pDisp = m_pSD.DisplayTransformation
		dPtSize = m_dPtSize
		dRefScale = pDisp.ReferenceScale
		If dRefScale <> 0 Then
			dMapScale = pDisp.ScaleRatio
			dPtSize = dPtSize * dRefScale / dMapScale
		End If
		dPtSize = dPtSize + 4

		' Convert to map coordinates and build envelope

		Dim dExt As Double, dX As Double, dY As Double
		Dim pEnv As IEnvelope

		dExt = pDisp.FromPoints(dPtSize)
		dX = pPoint.X
		dY = pPoint.Y
		pEnv = New Envelope
		pEnv.PutCoords(dX - dExt / 2, dY - dExt / 2, dX + dExt / 2, dY + dExt / 2)

		' Invalidate

		m_pSD.Invalidate(pEnv, False, esriScreenCache.esriNoScreenCache)

	End Sub

	Private Function TestComPort(ByVal sPortName As String) As Boolean

		Dim pFeedMan As IRealTimeFeedManager
		Dim pFeed As IGpsFeed
		Dim pConn As IGpsConnection
		Dim iPort As Short
		Dim eStatus As esriGpsConnectionStatus
		Dim bResult As Boolean

		pFeedMan = New RealTimeFeedManagerClass
		pFeed = pFeedMan.GpsFeed
		pConn = pFeed.Connection
		iPort = CShort(Strings.Right(sPortName, sPortName.Length - 3))
		pConn.CommPort = iPort
		pConn.BaudRate = 4800
		pConn.DataBits = 8
		pConn.Parity = esriGpsParity.esriGpsParityNone
		pConn.StopBits = esriGpsStopBits.esriGpsStopBits1
		bResult = True
		Try
			eStatus = pFeed.TestStatus
		Catch ex As Exception
			bResult = False
		End Try
		pConn = Nothing
		pFeed = Nothing
		pFeedMan = Nothing
		GC.Collect()
		If bResult = False Then
			Return False
		End If
		If eStatus = esriGpsConnectionStatus.esriGpsConnectionStatusReceiving Then
			Return True
		ElseIf eStatus = esriGpsConnectionStatus.esriGpsConnectionStatusPoorSignal Then
			Return True
		End If

		Return False

	End Function

End Class